/* libStreamedXML/src/lib/Core/Decoder.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lsx {



/*! \brief Basic UTF-8 character decoder.

This is a simple implementation of the character decoder layer. It takes
in a pure UTF-8 byte stream and outputs characters to the parser layer.
Any exceptions thrown from the above layers will simply be propagated to
the caller of feedData().

The decoder is stateless with respect to the Streamed XML stream, but stateful with respect to the
UTF-8 byte stream. It can be reset with a call to restart().

*/
class Decoder {
public:
    /*! \brief Constructor.

    \param parser The Streamed XML parser which will receive decoded
        characters.
    \param decoder If you wish to use a more complex decoder, you may pass it here. If you pass 0,
        the default UTF-8 decoder will be used.

    Sets up the finite state machine and records the Streamed XML parser
    in use. Doesn't generate any data.
    
    On deletion, \a parser will not be informed or deleted, but \a decoder will be deleted.

    */
    Decoder(Parser* parser, utf8::Decoder* decoder = 0);

    /// Destructor (frees \a decoder passed to ctor).
    virtual ~Decoder();



    /*! \brief Feed data to decoder.

    \param bytes The raw input bytes.
    \param amount The number of input bytes (may be zero).
    \throws (anything) Any exceptions from the Parser layer.
    \throws utf8::Error on UTF-8 decoding errors.

    This function is called whenever raw input data becomes available.
    It decodes a UTF-8 byte stream into characters, which it then passes
    to the Parser layer.
    
    */
    virtual void feedData(const char* bytes, size_t amount);



    /// Restart encoder if it is pointed at a new byte stream, etc.
    void restart();
    
private:
    Parser* parser;
    utf8::Decoder* decoder;
    static const char* sxmlRestartSequence;
    int sxmlRestartCount;
};



}
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
