/* libStreamedXML/src/lib/Core/Exceptions.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lsx {



/*! \brief Exception base class.

This class is the base class of all exceptions. It simply provides a mechanism for storing a
message; more specific details must be stored in base classes.

*/
class Exception : public std::exception {
public:
    /*! \brief Constructor.

    \param reason Reason for the exception.
    \param line Line number of input (starting from 0).
    \param col Column of input (starting from 0).

    The constructor simply stores the reason for the exception, which may be later accessed for
    reporting to the user.

    */
    Exception(const std::wstring& reason, int line, int col);
               
    /// Destructor.
    virtual ~Exception() throw()
    { }
    
    /// Find what caused the error.
    virtual const char* what() const throw();

    /// Reason for the exception.
    const std::wstring reason;

    /// Line on which the exception occurred (starting from 0).
    const int line;

    /// Column on which the exception occurred (starting from 0).
    const int col;
    
private:
    mutable std::string utf8Reason;
};



/// Thrown when XML is not well formed.
class NotWellFormed : public Exception {
private:
    static std::wstring format(const std::wstring& error, int line, int col);

public:
    /*! \brief Constructor.

    \param error Reason for the error.
    \param line Line on which the exception occurred.
    \param col Column on which the exception occurred.

    The constructor will store the reason for the error. It will also prepare a suitable message
    for the Exception base class.

    */
    NotWellFormed(const std::wstring& error, int line, int col);
    
    /// Destructor.
    virtual ~NotWellFormed() throw()
    { }

    /// Reason for the error.
    const std::wstring error;
};



/// Thrown when an unknown entity is referred to.
class UnknownEntity : public Exception {
private:
    static std::wstring format(const std::wstring& name, int line, int col);

public:
    /*! \brief Constructor.

    \param name Name of the unknown entity.
    \param line Line on which the exception occurred.
    \param col Column on which the exception occurred.
    
    The constructor will store the name of the unknown entity. It will also prepare a suitable
    message for the Exception base class.

    */
    UnknownEntity(const std::wstring& name, int line, int col);
    
    /// Destructor.
    virtual ~UnknownEntity() throw()
    { }

    /// Name of unknown entity.
    const std::wstring name;
};



}
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
